@@ -614,11 +614,34 @@ public:
   void CycleActiveWindow();
   uint32_t GetCapturableWindowCount();
 
+  bool GetTrackedFileData(const rdcstr &nickname, bytebuf &data) const;
+  bool AddTrackedFileReference(const rdcstr &nickname, const rdcstr &filepath);
+  void ClearTrackedFiles();
+  bool HasTrackedFileData() const;
+  rdcarray<rdcstr> GetTrackedFileNicknames() const;
+
+  RDResult EmbedExternalFiles(RDCFile *rdc);
+  RDResult RemoveExternalFiles(RDCFile *rdc);
+  bool HasEmbeddedFiles(RDCFile *rdc) const;
+  RDResult ReadExternalFiles(RDCFile *rdc);
+
 private:
   RenderDoc();
   ~RenderDoc();
 
+  struct TrackedFile
+  {
+    TrackedFile() = default;
+    TrackedFile(const rdcstr &name, const rdcstr &path) : nickname(name), filepath(path) {}
+    TrackedFile(const rdcstr &name, const bytebuf &contents) : nickname(name), data(contents) {}
+    rdcstr nickname;
+    rdcstr filepath;
+    bytebuf data;
+  };
+
   void SyncAvailableGPUThread();
+  RDResult WriteExternalFiles(RDCFile *rdc, const rdcarray<TrackedFile> &trackedFiles);
+  bool DoesTrackedFileExist(const rdcstr &nickname) const;
 
   bool m_Replay;
 
@@ -720,6 +743,9 @@ private:
   ICrashHandler *m_ExHandler;
   Threading::RWLock m_ExHandlerLock;
 
+  mutable Threading::RWLock m_TrackedFilesLock;
+  rdcarray<TrackedFile> m_TrackedFiles;
+
   void ProcessConfig();
 
   SDObject *FindConfigSetting(const rdcstr &name);
