@@ -729,6 +729,7 @@ extern bool IsGLES;
   /* super-set.                                              */  \
   EXT_TO_CHECK(32, 31, ARB_texture_multisample_no_array)         \
   EXT_TO_CHECK(32, 32, ARB_texture_multisample)                  \
+  EXT_TO_CHECK(32, 99, ARB_depth_clamp)                          \
   EXT_TO_CHECK(33, 30, ARB_explicit_attrib_location)             \
   EXT_TO_CHECK(33, 30, ARB_sampler_objects)                      \
   EXT_TO_CHECK(33, 30, ARB_texture_swizzle)                      \
@@ -737,6 +738,7 @@ extern bool IsGLES;
   EXT_TO_CHECK(33, 30, ARB_shader_bit_encoding)                  \
   EXT_TO_CHECK(40, 32, ARB_draw_buffers_blend)                   \
   EXT_TO_CHECK(40, 31, ARB_draw_indirect)                        \
+  EXT_TO_CHECK(40, 32, ARB_texture_gather)                       \
   EXT_TO_CHECK(40, 32, ARB_gpu_shader5)                          \
   EXT_TO_CHECK(40, 32, ARB_sample_shading)                       \
   EXT_TO_CHECK(40, 99, ARB_shader_subroutine)                    \
@@ -759,6 +761,7 @@ extern bool IsGLES;
   EXT_TO_CHECK(43, 31, ARB_compute_shader)                       \
   EXT_TO_CHECK(43, 32, ARB_copy_image)                           \
   EXT_TO_CHECK(43, 30, ARB_ES3_compatibility)                    \
+  EXT_TO_CHECK(45, 31, ARB_ES3_1_compatibility)                  \
   EXT_TO_CHECK(43, 30, ARB_invalidate_subdata)                   \
   EXT_TO_CHECK(43, 99, ARB_internalformat_query2)                \
   EXT_TO_CHECK(43, 31, ARB_program_interface_query)              \
@@ -789,6 +792,7 @@ extern bool IsGLES;
   EXT_TO_CHECK(99, 99, EXT_texture_sRGB_decode)                  \
   EXT_TO_CHECK(99, 99, INTEL_performance_query)                  \
   EXT_TO_CHECK(99, 99, EXT_texture_buffer)                       \
+  EXT_TO_CHECK(99, 99, KHR_shader_subgroup)                      \
   /* OpenGL ES extensions */                                     \
   EXT_TO_CHECK(99, 32, EXT_color_buffer_float)                   \
   EXT_TO_CHECK(99, 99, EXT_color_buffer_half_float)              \
@@ -823,6 +827,7 @@ extern bool IsGLES;
   EXT_COMP_CHECK(ARB_base_instance, EXT_base_instance)                                      \
   EXT_COMP_CHECK(ARB_copy_image, EXT_copy_image)                                            \
   EXT_COMP_CHECK(ARB_copy_image, OES_copy_image)                                            \
+  EXT_COMP_CHECK(ARB_depth_clamp, EXT_depth_clamp)                                          \
   EXT_COMP_CHECK(ARB_draw_buffers_blend, EXT_draw_buffers_indexed)                          \
   EXT_COMP_CHECK(ARB_draw_buffers_blend, OES_draw_buffers_indexed)                          \
   EXT_COMP_CHECK(ARB_geometry_shader4, EXT_geometry_shader)                                 \
