@@ -56,8 +56,11 @@ void GLReplay::RenderMesh(uint32_t eventId, const rdcarray<MeshFormat> &secondar
   GLMarkerRegion renderMesh(
       StringFormat::Fmt("RenderMesh with %zu secondary draws", secondaryDraws.size()));
 
+  float nearPlane = cfg.cam ? ((Camera *)cfg.cam)->GetNear() : 0.1f;
+  float farPlane = cfg.cam ? ((Camera *)cfg.cam)->GetFar() : 100000.0f;
+
   Matrix4f projMat =
-      Matrix4f::Perspective(90.0f, 0.1f, 100000.0f, DebugData.outWidth / DebugData.outHeight);
+      Matrix4f::Perspective(90.0f, nearPlane, farPlane, DebugData.outWidth / DebugData.outHeight);
 
   Matrix4f camMat = cfg.cam ? ((Camera *)cfg.cam)->GetMatrix() : Matrix4f::Identity();
 
@@ -95,6 +98,8 @@ void GLReplay::RenderMesh(uint32_t eventId, const rdcarray<MeshFormat> &secondar
   }
 
   drv.glDisable(eGL_CULL_FACE);
+  if(HasExt[ARB_depth_clamp])
+    drv.glEnable(eGL_DEPTH_CLAMP);
 
   if(cfg.position.unproject)
   {
@@ -352,6 +357,8 @@ void GLReplay::RenderMesh(uint32_t eventId, const rdcarray<MeshFormat> &secondar
   if(cfg.visualisationMode != Visualisation::NoSolid && topo != eGL_PATCHES)
   {
     drv.glDepthFunc(eGL_LESS);
+    if(HasExt[ARB_depth_clamp])
+      drv.glDisable(eGL_DEPTH_CLAMP);
 
     GLuint solidProg = prog;
 
@@ -424,6 +431,9 @@ void GLReplay::RenderMesh(uint32_t eventId, const rdcarray<MeshFormat> &secondar
     drv.glDisableVertexAttribArray(1);
 
     drv.glUseProgram(prog);
+
+    if(HasExt[ARB_depth_clamp])
+      drv.glEnable(eGL_DEPTH_CLAMP);
   }
 
   drv.glDepthFunc(eGL_ALWAYS);
