@@ -58,6 +58,7 @@ ReplayController::ReplayController()
 
 ReplayController::~ReplayController()
 {
+  RenderDoc::Inst().UnregisterMemoryRegion(this);
   CHECK_REPLAY_THREAD();
 }
 
@@ -1971,6 +1972,7 @@ void ReplayController::Shutdown()
     m_pDevice->Shutdown();
   m_pDevice = NULL;
 
+  RenderDoc::Inst().ClearTrackedFiles();
   delete this;
 }
 
@@ -2011,6 +2013,8 @@ bool ReplayController::FatalErrorCheck()
     m_GLPipelineState = GLPipe::State();
     m_VulkanPipelineState = VKPipe::State();
 
+    m_PipeState.SetDescriptorAccess({}, {}, {});
+
     return true;
   }
 
